#include <iostream>
#include <vector>
#include <cmath>
#include "test_movement.h"
#include "test_freakture.h"
#include "test_freaktory.h"
#include "test_setup.h"

movement::movement()
{
	
}

movement::~movement()
{
	
}

void movement::SetRandomRot(freakture& SRR)
{
	SRR.SetRot((((float)rand()/RAND_MAX)*360)-180);	
}

int movement::CheckBoundaries(freakture& FsB)
{
	if(FsB.GetPosX()>0 && FsB.GetPosX()<WIDTH && FsB.GetPosY()>0 && FsB.GetPosY()<HEIGHT)
		return 0; //freakture is totally inside the boundaries
	else
		return 1; //freakture is outside both X and Y boundaries
}

void movement::SetHeading(freakture& FsH)
{
	FsH.SetHeadingX(cos(FsH.GetRot()*DEGTORAD)); //rot in rads = deg*(pi/180)
	FsH.SetHeadingY(sin(FsH.GetRot()*DEGTORAD));	
}

void movement::SetNextHeading(freakture& FsH)
{
	FsH.SetNextHeadingX(cos(FsH.GetRot()*DEGTORAD)); //rot in rads = deg*(pi/180)
	FsH.SetNextHeadingY(sin(FsH.GetRot()*DEGTORAD));	
}

void movement::SetNewPosition(freakture& FsP)
{
	//freakture will have a speed multiplyer thingy.  heading*speed*time
	//std::cout<<"time: "<<setup::current_time<<std::endl;
	FsP.SetPosX(FsP.GetPosX()+(FsP.GetHeadingX()*FsP.GetSpeed()*setup::current_time));
	FsP.SetPosY(FsP.GetPosY()+(FsP.GetHeadingY()*FsP.GetSpeed()*setup::current_time));
}


void movement::Restless(freakture& FsR)
{
	FsR.SetSpeed(0.2);
	if(((float)rand()/RAND_MAX)<0.001)
	{
		SetRandomRot(FsR);
		SetHeading(FsR);
	}	
}

void movement::Amble(freakture& FsAmb)
{
	float MaxTime = 3000.0;
	float t = 0;
	float length, x, y;
	FsAmb.SetSpeed(0.1);
	
	//std::cout<<"somthing"<<std::endl;
	
	if(((float)rand()/RAND_MAX)<0.005)
	{
		TimePassed = 0;
		FsAmb.SetLastHeadingX(FsAmb.GetHeadingX());
		FsAmb.SetLastHeadingY(FsAmb.GetHeadingY());
		SetRandomRot(FsAmb);
		SetNextHeading(FsAmb);
	}

	//need to set a lastheading.  interpolate between last and next, not current and next!

	if(FsAmb.GetHeadingX() != FsAmb.GetNextHeadingX() && FsAmb.GetHeadingY() != FsAmb.GetNextHeadingY())
	//if(FsAmb.GetNextHeadingX() != FsAmb.GetHeadingX() && FsAmb.GetNextHeadingY() != FsAmb.GetHeadingY())
	{
		TimePassed += setup::current_time;
		if(TimePassed>MaxTime)
			TimePassed=MaxTime;
			
		t=TimePassed/MaxTime;
		
		//std::cout<<"time: "<<t<<std::endl;
		
		//blending function P1+t(P2-P1)
		/*FsAmb.SetHeadingX*/x=(FsAmb.GetLastHeadingX()+t*(FsAmb.GetNextHeadingX()-FsAmb.GetLastHeadingX()));
		/*FsAmb.SetHeadingY*/y=(FsAmb.GetLastHeadingY()+t*(FsAmb.GetNextHeadingY()-FsAmb.GetLastHeadingY()));
		//length = sqrt((FsAmb.GetHeadingX()*FsAmb.GetHeadingX())+(FsAmb.GetHeadingY()*FsAmb.GetHeadingY()));
		//std::cout<<"x,y: "<<x<<", "<<y<<std::endl;
		length=sqrt((x*x)+(y*y));
		//std::cout<<"length: "<<length<<std::endl;
		//tx=FsAmb.GetHeadingX()/length;
		//ty=FsAmb.GetHeadingY()/length;
		//FsAmb.SetHeadingX(x/length);
		//FsAmb.SetHeadingY(y/length);
		x=x/length;
		y=y/length;
		FsAmb.SetHeadingX(x);
		FsAmb.SetHeadingY(y);
		//length=sqrt((x*x)+(y*y));
		//std::cout<<"length: "<<length<<std::endl;
	}
}

void movement::HeadToCentre(freakture& FsC)
{
	float length, vX, vY;
	//get the vector from freaktures position to the centre
	vX=CENTREX-FsC.GetPosX();
	vY=CENTREY-FsC.GetPosY();
	//set the heading to the normalised vector
	//to normalise, find the length of the vector => sqrt(x*x+y*y)
	//divide each component by the length.
	length=sqrt((vX*vX)+(vY*vY));
	//std::cout<<"length: "<<length<<std::endl;
	FsC.SetHeadingX(vX/length);
	FsC.SetHeadingY(vY/length);
	//check that it has normalised:
	//length=sqrt((FsC.GetHeadingX()*FsC.GetHeadingX())+(FsC.GetHeadingY()*FsC.GetHeadingY()));
	//std::cout<<"length: "<<length<<std::endl;
	//these didn't work :(
	//FsC.SetHeadingX(Normalise(0,vX,vY));
	//FsC.SetHeadingY(Normalise(1,vX,vY));
}

//do this later!
/*float movement::Normalise(int xory, float x, float y)
{
	//set the length.
	float length = sqrt((x*x)+(y*y));
	float tx, ty;
	
	std::cout<<"length= "<<length<<std::endl;
	
	tx=(x/length);
	ty=(y/length);
	
	length = sqrt((tx*tx)+(ty*ty));
	std::cout<<"length= "<<length<<std::endl;
	
	//set xory to 0 for setting x, and 1 for setting y
	if (xory == 0)
		return (x/length);
	else
		return (y/length);
	
	
}*/


void movement::UpdateMoves(std::vector<freakture>& Fs3)
{
	unsigned int i;
	
	for(i=0;i<Fs3.size();i++)
	{
		Fs3[i].SetLastPosX(Fs3[i].GetPosX());
		Fs3[i].SetLastPosY(Fs3[i].GetPosY());
		if(CheckBoundaries(Fs3[i])==0)
			//Restless(Fs3[i]);
			Amble(Fs3[i]);
		else
			HeadToCentre(Fs3[i]);
		SetNewPosition(Fs3[i]);
	}
}


